@@ -13,11 +13,11 @@ from mgetool.tool import time_this_function
 from sklearn.base import BaseEstimator
 from sklearn.metrics import get_scorer
 
-from fastgplearn.backend.p_numpy import p_np_str_name, p_np_cal, lg, lr
 from fastgplearn.backend import backend
+from fastgplearn.backend.p_numpy import p_np_str_name, p_np_cal, lg, lr
 from fastgplearn.gp import set_seed, generate_random, select_index, mutate_random, crossover, mutate_sci
-from fastgplearn.tools import Hall, Logs
 from fastgplearn.sci_formula import usr_preset
+from fastgplearn.tools import Hall, Logs
 
 try:
     import torch
@@ -50,19 +50,21 @@ class SymbolicEstimator(BaseEstimator, ABC):
             p_mutate: mutate probability.
             p_crossover (float): crossover probability.
             random_state (int):random state
-            hall_of_fame (int): hall of frame number.
+            hall_of_fame (int): hall of frame number to add to next generation.
+            store_of_fame (int): hall of frame number to return result.
             method_backend (str): optional: ("p_numpy","c_numpy","p_torch","c_torch")
             device (str): default "cpu", "cuda:0", only accessible of torch.
             func_p (np.ndarray,tuple): with shape (n_function,), probability values of each function.
             sci_preset (str,list): None, "default" or user self-defined list template, default None.
 
+
         """
 
         assert population_size > 100 and generations >= 1
-        # assert store_of_fame <= 0.01 * population_size
         assert tournament_size <= 30 and hall_of_fame <= store_of_fame
         assert hall_of_fame >= 1
         assert store_of_fame >= 10
+        assert depth[1] <= 8
         try:
             self.func_names, self._str_name, self._score_mp, self._score, self._cal = backend[method_backend]
         except KeyError:
@@ -132,7 +134,7 @@ class SymbolicEstimator(BaseEstimator, ABC):
 
         func_index_ = set(tuple(self.func_index))
         for site, pre_i in sci_preset:
-            if len(set(tuple(pre_i)) & func_index_) == 0:
+            if set(tuple(pre_i)) <= func_index_:
                 temp_sci_preset.append([list(pre_i), [self.func_index.index(i) for i in pre_i]])
 
         return temp_sci_preset
@@ -393,7 +395,8 @@ class SymbolicRegressor(SymbolicEstimator):
             p_mutate: mutate probability.
             p_crossover (float): crossover probability.
             random_state (int):random state
-            hall_of_fame (int): hall of frame number.
+            hall_of_fame (int): hall of frame number to add to next generation.
+            store_of_fame (int): hall of frame number to return result.
             method_backend (str): optional: ("p_numpy","c_numpy","p_torch","c_torch")
             device (str): default "cpu", "cuda:0", only accessible of torch.
             func_p (np.ndarray): with shape (n_function,), probability values of each function.
@@ -410,6 +413,7 @@ class SymbolicRegressor(SymbolicEstimator):
                                                 constant_range=constant_range, constants=constants,
                                                 device=device, n_jobs=n_jobs, verbose=verbose, func_p=func_p,
                                                 )
+        self.logs = Logs("Person Corr")
 
     @staticmethod
     def single_coef_linear(X, y):
@@ -461,7 +465,7 @@ class SymbolicRegressor(SymbolicEstimator):
         """
         return super().score(X, y, scoring, n=n)
 
-    def top_n(self, n, scoring="r2"):
+    def top_n(self, n=0, scoring="r2"):
         """Print the top n result. The best one is index 0.
 
         Args:
@@ -474,11 +478,18 @@ class SymbolicRegressor(SymbolicEstimator):
         func = scorer._score_func
         sign = scorer._sign
 
-        print(f"scoring by {scoring}: ( score, expression, coef, intercept )")
+        self.logs.record_and_print(" ")
+        self.logs.record_and_print(f"The top {n} result:")
+        self.logs.record_and_print(f"Scoring by {scoring}: ( score, expression, coef, intercept )")
 
-        for i in range(self.store_of_fame):
-            pre_y, coef_, intercept_ = self.single_cal(n=n, with_coef=True)
-            print(sign * func(self.y, pre_y), self.single_name(i), coef_, intercept_)
+        for ni in range(n):
+            pre_y, coef_, intercept_ = self.single_cal(n=ni, with_coef=True)
+            msg = str((sign * func(self.y, pre_y), self.single_name(ni), coef_, intercept_))
+            self.logs.record_and_print(msg)
+
+    def best_expression(self, scoring="r2"):
+        """Print the best expression."""
+        self.top_n(n=0, scoring=scoring)
 
 
 class SymbolicClassifier(SymbolicEstimator):
@@ -530,7 +541,8 @@ class SymbolicClassifier(SymbolicEstimator):
             p_mutate: mutate probability.
             p_crossover (float): crossover probability.
             random_state (int):random state
-            hall_of_fame (int): hall of frame number.
+            hall_of_fame (int): hall of frame number to add to next generation.
+            store_of_fame (int): hall of frame number to return result.
             method_backend (str): optional: ("p_numpy","c_numpy","p_torch","c_torch")
             device (str): default "cpu", "cuda:0", only accessible of torch.
             func_p (np.ndarray,tuple): with shape (n_function,), probability values of each function.
@@ -546,6 +558,7 @@ class SymbolicClassifier(SymbolicEstimator):
                                                  tournament_size=tournament_size, sci_preset=sci_preset,
                                                  constant_range=constant_range, constants=constants,
                                                  device=device, n_jobs=n_jobs, verbose=verbose, func_p=func_p, )
+        self.logs = Logs("Accuracy")
 
     def fit(self, X: np.ndarray, y: np.ndarray, xs_p: np.ndarray = None, x_label=None):
         """
@@ -638,9 +651,14 @@ class SymbolicClassifier(SymbolicEstimator):
         func = scorer._score_func
         sign = scorer._sign
 
-        print(f"The top result:")
-        print(f"scoring by {scoring}: ( score, expression, coef, intercept )")
+        self.logs.record_and_print(f"The top {n} result:")
+        self.logs.record_and_print(f"Scoring by {scoring}: ( score, expression, coef, intercept )")
 
-        for i in range(self.store_of_fame):
-            pre_y, coef_, intercept_ = self.single_cal(n=n, with_coef=True)
-            print(sign * func(self.y, self.cla(pre_y)), self.single_name(i), coef_, intercept_)
+        for ni in range(n):
+            pre_y, coef_, intercept_ = self.single_cal(n=ni, with_coef=True)
+            msg = str((sign * func(self.y, self.cla(pre_y)), self.single_name(ni), coef_, intercept_))
+            self.logs.record_and_print(msg)
+
+    def best_expression(self, scoring="accuracy"):
+        """Print the best expression."""
+        self.top_n(n=0, scoring=scoring)
